/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ex2;

/**
 * Couche métier de la calculette.
 *
 * @author rosmord
 */
public class CalculetteModele {

    /**
     * Le nombre qu'on est en train de saisir.
     */
    private String saisieEnCours = "";
    /**
     * La dernière valeur calculée.
     */
    private double valeurCalculee = 0;
    /**
     * La dernière opération en attente.
     */
    private char operationPrecedente = '+';

    public double getValeurCalculee() {
        return valeurCalculee;
    }

    /**
     * Ajoute un symbole à la saisie en cours.
     * <p>N'a de résultat que si le caratère est un chiffre ou '.'.
     * <p>Si le caractère est '.', il n'est ajouté que s'il n'est pas déjà
     * présent.
     *
     * @param c
     */
    public void ajouterSymbole(char c) {
        if (Character.isDigit(c)) {
            saisieEnCours += c;
        } 
    }

    /**
     * Retourne l'affichage théorique de la calculatrice. Si que la saisie en
     * cours est vide, on doit afficher la dernière valeur calculée.
     *
     * @return
     */
    public String getAffichage() {
        if ("".equals(saisieEnCours)) {
            return "" + valeurCalculee;
        } else {
            return saisieEnCours;
        }
    }

    public double getValeurAffichage() {
        return Double.parseDouble(getAffichage());
    }

    /**
     * Retourne la valeur en cours de saisie sous forme de nombre réel. Si la
     * valeur en question est vide, retourne 0.
     *
     * @return
     */
    protected double getNombreEnConstruction() {
        try {
            return Double.parseDouble(saisieEnCours);
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    /**
     * Entrée de l'opérateur multiplier.
     * <p> Provoque l'exécution de l'opération en attente.
     * <p> L'opération '*' doit devenir l'opération en attente.
     */
    public void multiplierOp() {
        effectuerOperation();
        operationPrecedente = '*';
    }

    /**
     * Entrée de l'opérateur additionner.
     * <p> Provoque l'exécution de l'opération en attente.
     * <p> L'opération '*' doit devenir l'opération en attente.
     */
    public void additionnerOp() {
        effectuerOperation();
        operationPrecedente = '+';
    }

    public void soustraireOp() {
        effectuerOperation();
        operationPrecedente = '-';
    }

    public void diviserOp() {
        effectuerOperation();
        operationPrecedente = '/';
    }

    /**
     * Entrée de l'opérateur egal.
     * <p> Provoque l'exécution de l'opération en attente.
     * <p> L'opération '=' doit devenir l'opération en attente.
     * <p> Quand elle sera effectuée, elle aura pour valeur le nombre en cours de saisie
     * (par exemple, si on tape "21 + 9 = 5 * 3 =", la valeur finale sera 15. 
     */
    public void egalOp() {
        effectuerOperation();
        operationPrecedente = '=';
    }

    /**
     * Effectue l'opération demandée.
     * <p> Effectue l'opération en attente entre la saisie en cours et la
     * précédente valeur calculée.
     * <p> Si la saisie en cours est vide, l'opération est sans effet.
     */
    private void effectuerOperation() {
        if ("".equals(saisieEnCours)) {
            return;
        }
        switch (operationPrecedente) {
            case '+':
                valeurCalculee = valeurCalculee + getNombreEnConstruction();
                break;
            case '*':
                valeurCalculee = valeurCalculee * getNombreEnConstruction();
                break;
            case '/':
                valeurCalculee = valeurCalculee / getNombreEnConstruction();
                break;
            case '-':
                valeurCalculee = valeurCalculee - getNombreEnConstruction();
                break;
            case '=':
                valeurCalculee = getNombreEnConstruction();
                break;
        }
        saisieEnCours = "";
    }
}
